@@ -22,9 +22,6 @@
 
 moments <- function(par, fam_name, what = "mean", ex_fun = NULL) {
 
-  # get rownames
-  rnames <- row.names(par)
-
   # Stop if neither gamlss nor bamlss
   if (!is.gamlss(fam_name) && !is.bamlss(fam_name))
     stop("This function only works for bamlss/gamlss models")
@@ -44,14 +41,18 @@ moments <- function(par, fam_name, what = "mean", ex_fun = NULL) {
     }
   }
 
+  if (!samples)
+    rnames <- row.names(par)
+
   # Try out if the external function works
   funworks <- FALSE # Only do the computation if the external function is specified correctly
   if (!is.null(ex_fun)) {
+    fun <- get(ex_fun, envir = .GlobalEnv)
     tryCatch({
       if (samples)
-        ex_f(as.list(par[[1]][1, ]), ex_fun, samples = samples) # List of dataframes
+        ex_f(as.list(par[[1]][1, ]), unquotedfun = fun) # List of dataframes
       if (!samples)
-        ex_f(as.list(par[1, ]), ex_fun, samples = samples) # dataframe
+        ex_f(as.list(par[1, ]), unquotedfun = fun) # dataframe
       funworks <- TRUE
     }, error = function(e) {
       stop("External function not specified correctly!")
@@ -83,8 +84,10 @@ moments <- function(par, fam_name, what = "mean", ex_fun = NULL) {
       moms_raw <- apply(par, 1, function(x) {
         ex <- do.call(fam_called$mean, args = as.list(x)) # Expected value, use do.call because gamlss doesnt have par as parameter but only the named parameters...
         vx <- do.call(fam_called$variance, args = as.list(x)) # Variance
-        ex_fun <- do.call(ex_fun, args = list(x)) # External function, where we input it as a list
-        return(c(Expected_Value = ex, Variance = vx, ex_fun = ex_fun))
+        ex_fun_vals <- do.call(ex_fun, args = list(x)) # External function, where we input it as a list
+        return_vec <- c(Expected_Value = ex, Variance = vx, ex_fun = ex_fun_vals)
+        names(return_vec)[names(return_vec) == "ex_fun"] <- ex_fun # Give it the name of the variable ex_fun
+        return(return_vec)
       })
     }
 
@@ -122,12 +125,15 @@ moments <- function(par, fam_name, what = "mean", ex_fun = NULL) {
 
       # If we have external function
       if (funworks) {
+
         # Get moments for each row of par
         moms_raw <- apply(par, 1, function(x) {
           ex <- fam_called$mean(as.list(x)) # Expected value
           vx <- fam_called$variance(as.list(x)) # Variance
-          ex_fun <- ex_fun(as.list(x))
-          return(c(Expected_Value = ex, Variance = vx, ex_fun = ex_fun))
+          ex_fun_vals <- fun(as.list(x))
+          return_vec <- c(Expected_Value = ex, Variance = vx, ex_fun = ex_fun_vals)
+          names(return_vec)[names(return_vec) == "ex_fun"] <- ex_fun
+          return(return_vec)
         })
       }
 
@@ -158,20 +164,23 @@ moments <- function(par, fam_name, what = "mean", ex_fun = NULL) {
 
       # If we have external function
       if (funworks) {
+
         # Get moments for each sample and each prediction
         moms_raw <- lapply(par, function(listparts) {
           apply(listparts, 1, FUN = function(x) {
             ex <- fam_called$mean(as.list(x)) # Expected value
             vx <- fam_called$variance(as.list(x)) # Variance
-            ex_fun <- ex_fun(as.list(x))
-            return(c(Expected_Value = ex, Variance = vx, ex_fun = ex_fun))
+            ex_fun_vals <- fun(as.list(x))
+            return_vec <- c(Expected_Value = ex, Variance = vx, ex_fun = ex_fun_vals)
+            names(return_vec)[names(return_vec) == "ex_fun"] <- ex_fun
+            return(return_vec)
           })
         })
 
         # Reshaping necessary
         moms_raw <- lapply(moms_raw, FUN = function(x) {
           x <- t(x)
-          colnames(x) <- c("Expected_Value", "Variance", "ex_fun")
+          colnames(x) <- c("Expected_Value", "Variance", ex_fun)
           return(x)
         })
       }
@@ -187,7 +196,7 @@ moments <- function(par, fam_name, what = "mean", ex_fun = NULL) {
       # Lower Limit
       if (what == "lowerlimit") {
         comp_res <- lapply(moms_raw, FUN = function(listparts) {
-          apply(listparts, 2, FUN = quantile, probs = 0.025, na.rm = TRUE)
+          apply(listparts, 2, FUN = quantile, probs = 0.05, na.rm = TRUE)
         })
         moms <- do.call("rbind", args = comp_res)
       }
@@ -195,7 +204,7 @@ moments <- function(par, fam_name, what = "mean", ex_fun = NULL) {
       # Upper Limit
       if (what == "upperlimit") {
         comp_res <- lapply(moms_raw, FUN = function(listparts) {
-          apply(listparts, 2, FUN = quantile, probs = 0.975, na.rm = TRUE)
+          apply(listparts, 2, FUN = quantile, probs = 0.95, na.rm = TRUE)
         })
         moms <- do.call("rbind", args = comp_res)
       }
